/*= Rohde&Schwarz NGMO Application Sample ===================================*/
/* LabWindows/CVI 5.5.1 Application Sample                                   */
/* Original Release: Mai 2002                                                */
/* By: Jiri Kominek                                                          */
/*                                                                           */
/* Modification History:                                                     */
/*                                                                           */
/*     Rev. 1.1, 05/2005, CVI 7.0                                            */
/*     By: Martin Krcmar, Elcom, a.s.                                        */
/*         Added descriptions and helps                                      */
/*         Added an errror checking macro for driver functions               */
/*         Bug fixed:                                                        */
/*            - applySettingsCallback                                        */
/*         New function:                                                     */
/*            - configControlHelp                                            */
/*         Modified function:                                                */
/*            - applySettingsCallback                                        */
/*                                                                           */
/*     Rev. 1.02, 10/2002, CVI 6.0                                           */
/*     By: Werner Klotzsche, Rohde & schwarz                                 */
/*                 - changed in Panal description                            */
/*                                                                           */
/*     Rev. 1.01, 06/2002, CVI 6.0                                           */
/*     By: Werner Klotzsche, Rohde & schwarz                                 */
/*                 - Applay settings: Pulse measurement state disabled       */
/*                                                                           */
/* Description: This sample shows how to use application example for both    */
/*              channel reading from rsngmo instrument driver.               */
/*===========================================================================*/

/* Don't forget to load the instrument driver before using this sample project!  */

/* REVISION HISTORY

Revision 1.1 05/2005  */

#include <visa.h> 
#include <utility.h>
#include <ansi_c.h>
#include <cvirte.h>		
#include <userint.h>
#include "rsngmo.h"
#include "rsngmo_example.h"

/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        rsngmo_error_message (instrumentHandle, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", error_buffer); \
            rsngmo_error_query (instrumentHandle, &error, error_message);\
            sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetWaitCursor (0); \
            MessagePopup ("Error", error_buffer); \
			rsngmo_close(instrumentHandle); \
			break; \
        } \
    }

static int init;
static int panelHandle;
static int config;
ViInt32 gmeasure = 0; /* global for running the measurement loop */
ViInt32 gsamplesA, gsamplesB;

ViSession io;
ViStatus	error;
ViChar		error_message[256];
ViChar		error_buffer[256];

ViSession instrumentHandle;
ViReal64* pbufferChannelA;
ViReal64* pbufferChannelB;

int plotHandleA, plotHandleB, oldPlotHandleA = 0, oldPlotHandleB = 0;

int CVICALLBACK mainCallback (int panelOrMenuBarHandle, int controlOrMenuItemID,
                              int event, void *callbackData, int eventData1, int eventData2);

/*= Main Function ===========================================================*/
int main (int argc, char *argv[])
{
	int status;
	
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	if ((init = LoadPanel (0, "rsngmo_example.uir", INIT)) < 0)
		return -1;
		
    
	DisplayPanel (init);
    status = RunUserInterface ();
	DiscardPanel (init);
	
	if (status)
	    return 0;
	
	if ((panelHandle = LoadPanel (0, "rsngmo_example.uir", PANEL)) < 0)
		return -1;
	if ((config = LoadPanel (0, "rsngmo_example.uir", CONFIG)) < 0)
		return -1;
	
    // initialize data buffers
    pbufferChannelA = (ViReal64*) malloc (100 * sizeof(ViReal64));
    pbufferChannelB = (ViReal64*) malloc (100 * sizeof(ViReal64));
	
	// install main callback for measuring
    if (InstallMainCallback (mainCallback, 0, 1))
        return -1; /* unable to install main callback*/
	
	DisplayPanel (panelHandle);
	RunUserInterface();
	DiscardPanel (panelHandle);
	DiscardPanel (config);
    
	// cleanup buffers
	free (pbufferChannelA);
	free (pbufferChannelB);
	
	do
	{
		CHECKERR(rsngmo_close (instrumentHandle) );
	} while (0);
	 
	
	return 0;
}

/*===========================================================================*/
/* Function: Panel Callback                                                  */
/*===========================================================================*/
int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_GOT_FOCUS:

			break;
		case EVENT_LOST_FOCUS:

			break;
		case EVENT_CLOSE:
			QuitUserInterface (0);
			break;
		}
	return 0;
}

/*===========================================================================*/
/* Function: Initialize	                                                     */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset.                                                      */
/*===========================================================================*/
int CVICALLBACK initInstrument (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
    ViChar instrDescr[256], errorMessage[256], errBuffer[256];
    ViInt16 addr;
    ViInt32 interface;
    ViBoolean id, rst;
    ViStatus error;
    
	switch (event)
		{
		case EVENT_COMMIT:
            SetWaitCursor(1);
            GetCtrlVal (init, INIT_INTERFACE, &interface);
            GetCtrlVal (init, INIT_ADDRESS, &addr);
            GetCtrlVal (init, INIT_ID, &id);
            GetCtrlVal (init, INIT_RST, &rst);
			switch (interface){
				case 0:
					sprintf (instrDescr, "%s%d", "GPIB::", addr);
					break;
				case 1:
					sprintf (instrDescr, "ASRL1");
					break;
				case 2:
					sprintf (instrDescr, "ASRL2");
					break;
			}		
			// performing the initialization
			if ((error = rsngmo_init (instrDescr, id, rst, &instrumentHandle)) < 0){
				rsngmo_error_message (instrumentHandle, error, errorMessage);
				sprintf (errBuffer,
						 "Initialization Error:\n\n%s\n\nCheck your connections and make sure you have the right GPIB address.",
						 errorMessage);
				MessagePopup ("ERROR!", errBuffer); 
                SetWaitCursor(0);
                return 0;
            }
			
			// configuring the front panel display
			CHECKERR( rsngmo_configureFrontPanel (instrumentHandle, VI_TRUE, RSNGMO_CHANNEL_A));
			CHECKERR( rsngmo_configureFrontPanel (instrumentHandle, VI_TRUE, RSNGMO_CHANNEL_B));    
            
            SetWaitCursor(0);
            QuitUserInterface(0);
			break;
		}
	return 0;
}

/*===========================================================================*/
/* Function: Exit                                                            */
/* Purpose:  This is a callback function of Exit button. It closes the       */
/*           session and exits the example.                                  */
/*===========================================================================*/
int CVICALLBACK cancel (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			QuitUserInterface (1);
			break;
		}
	return 0;
}

/*===========================================================================*/
/* Function: Configure                                                       */
/* Purpose:  This is a callback function of Configure button. It opens the   */
/*           window to configure the measurement.                            */
/*===========================================================================*/
int CVICALLBACK configureCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
            DisplayPanel (config);
			break;
		}
	return 0;
}

/*===========================================================================*/
/* Function: Start                                                           */
/* Purpose:  This is a callback function of Start button.                    */
/*===========================================================================*/
int CVICALLBACK measureCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle, PANEL_CONFIG, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle, PANEL_MEASURE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle, PANEL_STOP_MEASURE, ATTR_DIMMED, 0);
            gmeasure = 1;
			break;
		}
	return 0;
}

/*===========================================================================*/
/* Function: Stop                                                            */
/* Purpose:  This is a callback function of Stop button.                     */
/*===========================================================================*/
int CVICALLBACK stopMeasureCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
        {
		case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle, PANEL_CONFIG, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle, PANEL_MEASURE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle, PANEL_STOP_MEASURE, ATTR_DIMMED, 1);
            gmeasure = 0;
			break;
        }
	return 0;
}

/*===========================================================================*/
/* Function: Apply Settings                                                  */
/* Purpose:  This is a callback function of Apply button. The values are     */
/* 			 sent to the instrument                                          */
/*===========================================================================*/
int CVICALLBACK applySettingsCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	ViReal64 voltageA, voltageB, voltagePA, voltagePB;
	ViReal64 currentLimitA, currentLimitB;
	ViReal64 intervalA, intervalB;
	ViInt32 currentA, currentB;
	ViBoolean typeA, typeB;
	switch (event)
        {
		case EVENT_COMMIT:
		    // get control values from panel
            GetCtrlVal (panel, CONFIG_Voltage_A, &voltageA);
            GetCtrlVal (panel, CONFIG_Voltage_B, &voltageB);
            GetCtrlVal (panel, CONFIG_Voltage_P_A, &voltagePA);
            GetCtrlVal (panel, CONFIG_Voltage_P_B, &voltagePB);
            GetCtrlVal (panel, CONFIG_Current_limit_A, &currentLimitA);
            GetCtrlVal (panel, CONFIG_Current_limit_B, &currentLimitB);
            GetCtrlVal (panel, CONFIG_Type_A, &typeA);
            GetCtrlVal (panel, CONFIG_Type_B, &typeB);
            GetCtrlVal (panel, CONFIG_Current_A, &currentA);
            GetCtrlVal (panel, CONFIG_Current_B, &currentB);
            GetCtrlVal (panel, CONFIG_Interval_A, &intervalA);
            GetCtrlVal (panel, CONFIG_Interval_B, &intervalB);
            GetCtrlVal (panel, CONFIG_Samples_A, &gsamplesA);
            GetCtrlVal (panel, CONFIG_Samples_B, &gsamplesB);
            
            // if one or both voltages are set higher than 5V
            if ((voltageA - 5.0 > 1.0e-6) | (voltageB - 5.0 > 1.0e-6))
            {
            	// both Current Limits are coerced to max 2.5
            	if (currentLimitA - 2.5 > 1.0e-6) currentLimitA = 2.5;
            	if (currentLimitB - 2.5 > 1.0e-6) currentLimitB = 2.5;
            	// set the new values into the panel
            	SetCtrlVal (panel, CONFIG_Current_limit_A, currentLimitA);
            	SetCtrlVal (panel, CONFIG_Current_limit_B, currentLimitB);
            }
            
            // reallocate buffers
            pbufferChannelA = realloc (pbufferChannelA, gsamplesA * sizeof(ViReal64));
            pbufferChannelB = realloc (pbufferChannelB, gsamplesB * sizeof(ViReal64));
            
            // set parameters of channel A
            CHECKERR( rsngmo_configureOutput (instrumentHandle, RSNGMO_CHANNEL_A, VI_TRUE,
                                    RSNGMO_OUTP_BAND_HIGH, 0.00));
            
            // configuring the pulse trigger
            CHECKERR( rsngmo_configurePulseTrigger (instrumentHandle, RSNGMO_CHANNEL_A,
                                          RSNGMO_TRIG_SOURCE_INT,
                                          RSNGMO_TRIG_RANG_HIGH, 0.1, 1,
                                          RSNGMO_TRIG_SLOPE_POS, -50, 10));
            
            // configuring the source voltage and the current limit mode and value
            CHECKERR( rsngmo_configureSource (instrumentHandle, RSNGMO_CHANNEL_A, voltageA,
                                    currentLimitA, RSNGMO_SOUR_CURR_TYPE_LIMIT));
            
            // configures the voltage protection value and the max. settings
			// for voltage and current limit
            CHECKERR( rsngmo_configureSourceProtection (instrumentHandle, RSNGMO_CHANNEL_A,
                                              voltagePA, 15.0, 5.0));
			
			// configuring the measurement
			CHECKERR( rsngmo_configureMeasurement (instrumentHandle, RSNGMO_CHANNEL_A,
			    						 RSNGMO_FUNC_AVERAGE,
										 currentA, 1, 0.002));
            
            // configuring the pulse measurement
            CHECKERR( rsngmo_configurePulseMeasurement (instrumentHandle, RSNGMO_CHANNEL_A,
											  typeA ? RSNGMO_PULSE_CHAN_CURR:RSNGMO_PULSE_CHAN_DVM,
											  RSNGMO_FUNC_AVERAGE, gsamplesA,
											  intervalA, VI_FALSE));
        	
            // set parameters of channel B
            CHECKERR( rsngmo_configureOutput (instrumentHandle, RSNGMO_CHANNEL_B, VI_TRUE,
                                    RSNGMO_OUTP_BAND_HIGH, 0.00));
            
            // configuring the pulse trigger
            CHECKERR( rsngmo_configurePulseTrigger (instrumentHandle, RSNGMO_CHANNEL_B,
                                          RSNGMO_TRIG_SOURCE_INT,
                                          RSNGMO_TRIG_RANG_HIGH, 0.1, 1,
                                          RSNGMO_TRIG_SLOPE_POS, -50, 10));
            
            // configuring the source voltage and the current limit mode and value
            CHECKERR( rsngmo_configureSource (instrumentHandle, RSNGMO_CHANNEL_B, voltageB,
                                    currentLimitB, RSNGMO_SOUR_CURR_TYPE_LIMIT));
            
            // configures the voltage protection value and the max. settings
			// for voltage and current limit
            CHECKERR( rsngmo_configureSourceProtection (instrumentHandle, RSNGMO_CHANNEL_B,
                                              voltagePB, 15.0, 5.0));
			
			// configuring the measurement
			CHECKERR( rsngmo_configureMeasurement (instrumentHandle, RSNGMO_CHANNEL_B,
			    						 RSNGMO_FUNC_AVERAGE,
										 currentB, 1, 0.002));
            
            // configuring the pulse measurement
            CHECKERR( rsngmo_configurePulseMeasurement (instrumentHandle, RSNGMO_CHANNEL_B,
											  typeB ? RSNGMO_PULSE_CHAN_CURR:RSNGMO_PULSE_CHAN_DVM,
											  RSNGMO_FUNC_AVERAGE, gsamplesB,
											  intervalB, VI_FALSE));
        	// set graphs Y axes description
            SetCtrlAttribute (panelHandle, PANEL_Graph_A, ATTR_YNAME, typeA ? "Current" : "Voltage");
            SetCtrlAttribute (panelHandle, PANEL_Graph_B, ATTR_YNAME, typeB ? "Current" : "Voltage");
			
			HidePanel(panel);
            SetCtrlAttribute (panelHandle, PANEL_MEASURE, ATTR_DIMMED, 0);
			break;
        }
	return 0;
}

/*===========================================================================*/
/* Function: Main Callback                                                   */
/* Purpose:  This function performs the measurement and plots the data into  */
/*           the graphs.                                                     */
/*===========================================================================*/
int CVICALLBACK mainCallback (int panelOrMenuBarHandle, int controlOrMenuItemID,
                              int event, void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_IDLE:
            if (gmeasure)
            {
                ViInt32 countA = 100,
                		countB = 100;
                ViInt32 measStatus = 0;
                double	intervalA,
                		intervalB;
                ViBoolean measAReady = 0,
                          measBReady = 0;
                ViChar bufferA[2048], bufferB[2048];
                
                // trigger both channels
                CHECKERR( rsngmo_trigger (instrumentHandle, RSNGMO_CHANNEL_BOTH));
                
                // wait for finishing the measurement (see status system in the manual)
            	do {
            		CHECKERR( rsngmo_getStatusRegister (instrumentHandle, RSNGMO_REG_MEAS_COND,
            							      &measStatus));
            		
            		measAReady |= measStatus & 0x020;
            		measBReady |= measStatus & 0x100;
            		} while (!(measAReady || measBReady));

	            if (measAReady) 
            	{						    
                    // fetch the Channel A data
                    CHECKERR( rsngmo_fetchMeasurementArray (instrumentHandle, RSNGMO_CHANNEL_A, gsamplesA,
                    								pbufferChannelA, &countA));
		            GetCtrlVal (config, CONFIG_Interval_A, &intervalA);
		            
					oldPlotHandleA = plotHandleA;
					// plot measured data into the graph A
					plotHandleA = PlotWaveform (panelHandle, PANEL_Graph_A, pbufferChannelA, countA,
								  VAL_DOUBLE, 1.0, 0.0, 0.0, intervalA, VAL_THIN_LINE,
								  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
                    if (oldPlotHandleA)
                        DeleteGraphPlot (panelHandle, PANEL_Graph_A, oldPlotHandleA,
                                         VAL_IMMEDIATE_DRAW);
					
           		}
            		
	            if (measBReady) 
            	{
	            	// fetch the Channel B data
              	    CHECKERR( rsngmo_fetchMeasurementArray (instrumentHandle, RSNGMO_CHANNEL_B, gsamplesB,
                                 					pbufferChannelB, &countB));
              
	            	GetCtrlVal (config, CONFIG_Interval_B, &intervalB);    
					oldPlotHandleB = plotHandleB;
					// plot measured data into the graph B
					plotHandleB = PlotWaveform (panelHandle, PANEL_Graph_B, pbufferChannelB, countB,
								  VAL_DOUBLE, 1.0, 0.0, 0.0, intervalB, VAL_THIN_LINE,
								  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
                    if (oldPlotHandleB) 
                        DeleteGraphPlot (panelHandle, PANEL_Graph_B, oldPlotHandleB,
                                         VAL_IMMEDIATE_DRAW);
               	}
                
            }
            break;
        }
    return 0;
}

/*===========================================================================*/
/* Function: Control Help                                                    */
/* Purpose:  It displays a help window describing control being used.        */
/*===========================================================================*/
int CVICALLBACK controlHelp (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	ViChar *helpString = NULL;

	switch (event)
		{
		case EVENT_RIGHT_CLICK:
			switch (control)
				{
				case INIT_INTERFACE:
					helpString = "This control selects the communication interface\n"
										  "used to communicate with the instrument.";
				break;
				case INIT_ID:
					helpString = "This control specifies if an ID Query is sent to the\n"
                                    "instrument during the initialization procedure.\n\n"
                                    "Valid Range:\n"
                                    "VI_TRUE  (1) - Do Query (Default Value)\n"
                                    "VI_FALSE (0) - Skip Query";
				break;
				case INIT_ADDRESS:
					helpString = "This control selects the address of the device that is to be\n"
									"initialized.\n\n"
									"Valid Range:  Address 1 to 31\n\n"
									"Default Value:  Address 5";
					
				break;
				case INIT_RST:
					helpString = "This control specifies if the instrument is to be reset to its\n"
                                    "power-on settings during the initialization procedure.\n\n"
                                    "Valid Range:\n"
                                    "VI_TRUE  (1) - Reset Device (Default Value)\n"
                                    "VI_FALSE (0) - Don't Reset\n";
				
				break;
				}
				if (helpString) MessagePopup ("Help", helpString);
				
			break;
		}
	return 0;
}

/*===========================================================================*/
/* Function: Configuration Control Help                                      */
/* Purpose:  It displays a help window describing control being used.        */
/*===========================================================================*/
int CVICALLBACK configControlHelp (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	ViChar *helpString = NULL;

	switch (event)
		{
		case EVENT_RIGHT_CLICK:
			switch (control)
				{
				case CONFIG_Type_A:
				case CONFIG_Type_B:
					helpString = "For DVM measurement please use the DVM inputs!\n";
				break;
				case CONFIG_Voltage_A:
				case CONFIG_Voltage_B:
					helpString = "This control specifies the source voltage amplitude.\n\n"
									"Valid Range: 0.000 to 15.000 V\n\n"
									"Default Value: 5.000 V";
				break;
				case CONFIG_Voltage_P_B:
				case CONFIG_Voltage_P_A:
					helpString = "This control sets the source voltage protection.\n\n"
									"Valid Range: 1.5 to 22 V\n\n"
									"Default Value: 6.000 V";
				break;
				case CONFIG_Current_limit_B:
				case CONFIG_Current_limit_A:
					helpString = "This control sets the source current limit.\n\n"
									"Valid Range: 0.000 to 5.000 A \n(for voltage above 5V max. 2.5A)\n\n"
									"Default Value: 5.000A";
				break;
				case CONFIG_Current_B:
				case CONFIG_Current_A:
					helpString = "This control specifies the current measurement range.\n\n"
									"Valid Range: \nHigh (5A)\nMedium (500 mA)\nLow (5mA)\nAuto Range\n\n"
									"Default Value: High (5A)";
				break;
				case CONFIG_Interval_B:
				case CONFIG_Interval_A:
					helpString = "This control specifies the pulse measurement \nsample interval in seconds.\n\n"
									"Valid Range: 1.0E-5 to 1.00000\n\n"
									"Default Value: 0.001";
				break;
				case CONFIG_Samples_B:
				case CONFIG_Samples_A:
					helpString = "This control specifies the number of samples \n(pulse measurement sample length).\n\n"
									"Valid Range: 50 to 5000\n\n"
									"Default Value: 100";
				break;
				}
				if (helpString) MessagePopup ("Help", helpString);
				
			break;
		}
	return 0;
}
